
from ekp_sdk.db.contract_logs_repo import ContractLogsRepo
from ekp_sdk.db.contract_transactions_repo import ContractTransactionsRepo
from ekp_sdk.db.mg_client import MgClient
from ekp_sdk.db.pg_client import PgClient


from ekp_sdk.util.clean_null_terms import clean_null_terms


def Form(
    name, 
    schema, 
    children, 
    class_name=None,
    multi_record = None,
    on_submit = None,
    style=None,
    ):
    return {
        "_type": "Form",
        "props": clean_null_terms({
            "children": children,
            "className": class_name,
            "multiRecord": multi_record,
            "name": name,
            "onSubmit": on_submit,
            "schema": schema,
            "style": style
        })
    }




from ekp_sdk.ui.rpc.commify import *
from ekp_sdk.ui.rpc.format_mask_address import *
from ekp_sdk.ui.rpc.navigate import *
from ekp_sdk.ui.rpc.remove_form_record import *
from ekp_sdk.ui.rpc.sum import *
from ekp_sdk.ui.rpc.count import *


import json
import time

import aiohttp_cors
import socketio
from aiohttp import web


class ClientService:
    def __init__(self, port, plugin_id):
        self.port = port
        self.plugin_id = plugin_id
        self.sio = self.__get_socket_io()
        self.app = self.__get_web_app()
        self.sio.attach(self.app)
        self.controllers = []

    def __get_socket_io(self):
        sio = socketio.AsyncServer(
            async_mode='aiohttp', cors_allowed_origins='*')

        @sio.event
        async def connect(sid, environ, auth):
            print(f"✨ Client connected: {sid} => {len(self.controllers)} controllers")
            for controller in self.controllers:
                await controller.on_connect(sid)

        @sio.on('client-state-changed')
        async def on_client_state_changed(sid, data):
            print(f"📣 Client state changed: {sid} => {len(self.controllers)} controllers")
            for controller in self.controllers:
                await controller.on_client_state_changed(sid, json.loads(data))

        return sio

    def __get_web_app(self):
        app = web.Application()

        cors = aiohttp_cors.setup(app, defaults={
            "*": aiohttp_cors.ResourceOptions(
                allow_credentials=True,
                expose_headers="*",
                allow_methods="*",
                allow_headers="*",
            )
        })

        cors.add(app.router.add_static('/static', 'static'))

        return app

    def listen(self):
        web.run_app(self.app, port=self.port)

    def add_controller(self, controller):
        self.controllers.append(controller)

    async def emit_busy(self, sid, collection_name):
        layer = {
            "id": f"busy-{collection_name}",
            "collectionName": "busy",
            "set": [{"id": collection_name, }],
            "timestamp": int(time.time())
        }
        await self.emit_add_layers(sid, [layer])

    async def emit_done(self, sid, collection_name):
        message = {
            "query": {
                "id": f'busy-{collection_name}',
            }
        }
        await self.sio.emit('remove-layers', json.dumps(message), room=sid)

    async def emit_menu(self, sid, icon, title, nav_link, aliases=None, short_link=None, order=None, id=None):
        """
        Sends menu layer from server to the client side
        """

        if not id:
            id = f"{self.plugin_id}_{nav_link}"
            
        layer = {
            "id": id,
            "collectionName": "menus",
            "set": [
                {
                    "id": f"{self.plugin_id}_{nav_link}",
                    "icon": icon,
                    "title": title,
                    "navLink": nav_link,
                    "aliases": aliases,
                    "shortLink": short_link,
                    "order": order
                }
            ],
            "timestamp": int(time.time())
        }

        await self.emit_add_layers(sid, [layer])

    async def emit_page(self, sid, path, element):
        """
        Sends main page content from server to the client side
        """

        layer = {
            "id": f"{self.plugin_id}_page_{path}",
            "collectionName": "pages",
            "set": [
                {
                    "id": path,
                    "element": element
                }
            ],
            "timestamp": int(time.time())
        }

        await self.emit_add_layers(sid, [layer])

    async def emit_documents(self, sid, collection_name, documents, layer_id=None):

        if layer_id == None:
            layer_id = f"{self.plugin_id}_{collection_name}"
            
        layer = {
            "id": layer_id,
            "collectionName": collection_name,
            "set": documents,
            "timestamp": int(time.time())
        }

        await self.emit_add_layers(sid, [layer])

    async def emit_add_layers(self, sid, layers):
        message = {
            "layers": layers
        }

        await self.sio.emit('add-layers', json.dumps(message), room=sid)



from typing import List, TypedDict


class EtherscanLogDto(TypedDict, total=False):
    '''
    :example
    {
        "address": "0xe561479bebee0e606c19bb1973fc4761613e3c42",
        "topics": [
            "0xddf252ad1be2c89b69c2b068fc378daa952ba7f163c4a11628f55a4df523b3ef",
            "0x000000000000000000000000730e2065b9daee84c3003c05bf6d2b3a08e55667",
            "0x000000000000000000000000d7d19938eae260d7f0e0a4c36e665ff4cf4b7acc"
        ],
        "data": "0x000000000000000000000000000000000000000000000000076cd96f53f24b0a",
        "blockNumber": "0x4c3326",
        "timeStamp": "0x602e9ef1",
        "gasPrice": "0x2540be400",
        "gasUsed": "0x1b0f2",
        "logIndex": "0xf7",
        "transactionHash": "0x73844fcfc6beab2e973a897c9573f4d79811b12213ce263045a203e0d3cea90e",
        "transactionIndex": "0xb9"
    }
    '''

    address: str
    topics: List[str]
    data: str
    blockNumber: str
    timeStamp: str
    gasPrice: str
    gasUsed: str
    logIndex: str
    transactionHash: str
    transactionIndex: str



from typing import List
from ekp_sdk.dto.moralis_nft_dto import MoralisNftDto
from ekp_sdk.services.limiter import Limiter
from ekp_sdk.services.rest_client import RestClient


class MoralisApiService:
    def __init__(
        self,
        api_key: str,
        rest_client: RestClient
    ):
        self.base_url = "https://deep-index.moralis.io/api/v2"
        self.api_key = api_key
        self.rest_client = rest_client
        self.limiter = Limiter(250, 10)
        
    # -----------------------------------------------------------------
    
    async def get_nfts_by_owner_and_token_address(
        self,
        owner_address: str,
        token_address: str,
        chain: str
    ) -> List[MoralisNftDto]:
        
        
        url = f"{self.base_url}/{owner_address}/nft/{token_address}?chain={chain}&format=decimal"

        result = await self.__get(url)
        
        

        return result
    
    # -----------------------------------------------------------------
    
    async def __get(self, url):
        try:
            await self.limiter.acquire()
            headers = {"X-API-Key": self.api_key}
            result = await self.rest_client.get(
                url, 
                lambda data, text: data["result"],
                headers=headers
            )
        finally:
            self.limiter.release()
        return result
